# Generated by Django 3.1.1 on 2020-10-01 04:02

from django.db import migrations, models
import eav.fields


class Migration(migrations.Migration):

    dependencies = [
        ('eav', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='value',
            name='value_m2m_enum',
            field=models.ManyToManyField(related_name='eav_m2m_values', to='eav.EnumValue'),
        ),
        migrations.AlterField(
            model_name='attribute',
            name='datatype',
            field=eav.fields.EavDatatypeField(choices=[('text', 'Text'), ('date', 'Date'), ('float', 'Float'), ('int', 'Integer'), ('bool', 'True / False'), ('object', 'Django Object'), ('enum', 'Single Choice'), ('m2m_enum', 'Multiple Choice')], max_length=16, verbose_name='Data Type'),
        ),
    ]
