# Generated by Django 3.1.1 on 2020-11-04 05:31

from django.db import migrations
import eav.fields


class Migration(migrations.Migration):

    dependencies = [
        ('eav', '0004_auto_20201002_1407'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='slug',
            field=eav.fields.EavSlugField(help_text='Short unique attribute label', max_length=256, verbose_name='Slug'),
        ),
        migrations.AlterUniqueTogether(
            name='attribute',
            unique_together={('slug', 'tag')},
        ),
    ]
